# Manipulate olpc disk images.
#
# Copyright (C) 2007 Mitchell N Charity
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re

import util
from util import system_or_exception, getoutput_or_exception

class DiskImageManipulation:

    def laptop_image_partition_start_end_and_sizeMB(self,img_file):
        # could use file instead of fdisk
        info = getoutput_or_exception("/sbin/fdisk -ul "+img_file)
        #Device Boot      Start         End      Blocks   Id  System
        #xo-623.img 1        62     1899679      949809   83  Linux
        m = re.search('(\d+)\s+(\d)+\s+\d+\s+83\s+Linux',info)
        start_sector, end_sector = int(m.group(1)), int(m.group(2))
        start = start_sector * 512
        end = end_sector * 512
        size = end / 1024.0 / 1024.0
        return (start,end,size)

    def mount_img_file_on_dir(self,img_file,dir):
        offset, ignore, ignore = (
            self.laptop_image_partition_start_end_and_sizeMB(img_file))
        cmd = "/bin/mount -o loop,offset=%s %s %s"%(offset,img_file,dir)
        run_commands_as_root([cmd])
        
    def enlarge_disk_img(self,img_file,size_mb):
        offset, ignore, old_size = (
            self.laptop_image_partition_start_end_and_sizeMB(img_file))
        if old_size < size_mb:
            raise Exception("New size is smaller than old size: %d < %d"
                            %(new_size, olc_size))
        cmd = ("/bin/dd if=/dev/zero of=%s seek=%s obs=1MB count=0"
               %(img_file,size_mb))
        system_or_exception(cmd)

        cmds = ["/sbin/losetup /dev/loop0 %s"%(img_file),
                """perl -e '$|=1;for (split(//,"pdnp1  pw")){sleep(2);print "$_\\n";}sleep(2);'| /sbin/fdisk /dev/loop0""",
                "/sbin/losetup -o %s /dev/loop1 /dev/loop0"%(offset),
                "/sbin/fsck.ext3 -fy /dev/loop1",
                "/sbin/resize2fs /dev/loop1 %sM"%(int(0.95*int(size_mb))),
                "/sbin/losetup -d /dev/loop1",
                "/sbin/losetup -d /dev/loop0"]
        util.run_commands_as_root(cmds)

    def convert_img_bz2_to_qcow2(self,img_fn,cow_fn,tmp_dir=None):
        if not tmp_dir:
            tmp_dir = os.path.dirname(cow_fn)
        raw_fn = "%s/disk%d.raw"%(tmp_dir,os.getpid())
        try:
            system_or_exception("bzcat %s > %s"%(img_fn,raw_fn))
            self.convert_img_to_qcow2(raw_fn,cow_fn)
        finally:
            if os.path.isfile(raw_fn):
                os.unlink(raw_fn)

    def convert_img_to_qcow2(self,img_fn,cow_fn):
        cmd = "qemu-img convert %s -O qcow2 %s"%(img_fn,cow_fn)
        system_or_exception(cmd)
