# Obtain and cache OLPC disk images.
#
# Copyright (C) 2007 Mitchell N Charity
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re

import build_system
from util import system_or_exception, getoutput_or_exception


class ObtainBuilds:

    def __init__(self,cache_dir=None):
        self._cache_dir = cache_dir

        self._streams = build_system.OLPCStreams()

        self.cache = None
        if cache_dir:
            self.cache = CachedBuildDirectory(cache_dir)

    def download_build_cached(self,build):
        cached = self.cache.image_file_for_build(build)
        if cached:
            return cached
        else:
            return self.download_build(build)

    def download_build(self,build,dir=None):
        if not dir:
            dir = self._cache_dir
        if not dir:
            dir = "."

        img,md5 = build.urls_of_ext3_image_and_its_md5()
        img_fn = os.path.basename(img)
        img_path = os.path.join(dir,img_fn)
        md5_fn = img_fn + ".md5"

        #XXX fails if file already exists.  feature or bug?
        #XXX on failure, doesn't clean up
        system_or_exception("(cd %s; wget -nc -nv %s)"%(dir,md5))
        system_or_exception("(cd %s; wget -nc %s)"%(dir,img))
        print getoutput_or_exception("(cd %s; md5sum -c %s)"%(dir,md5_fn))

        return img_path


class CachedBuildDirectory:

    def __init__(self,directory):
        if not os.path.isdir(directory):
            raise Exception("Build cache directory doesn't exist: %s"%(dir))
        self.directory = directory
        self._index = None

    def builds(self):
        builds = []
        self._index = {}
        for filename in os.listdir(self.directory):
            try:
                build = build_system.Build.new_from_ext3_image_filename(
                    filename)
            except:
                continue
            builds.append(build)
            self._index[build.name()] = filename
        return builds

    def image_file_for_build(self,build):
        self.builds()
        name = build.name()
        if name in self._index:
            return os.path.join(self.directory,self._index[name])
        return None
